package SOAP::Transport::HTTP::Daemon::PreforkSOAP;

#use strict;
use vars qw(@ISA);
use SOAP::Transport::HTTP;
use POSIX ":sys_wait_h";

# Implementation and Idea by Michael Brown and Christopher Stanton
# Inspired by Peter Fraenkel (Peter.Fraenkel@...) and
# his ForkAfterProcessing module.

@ISA = qw(SOAP::Transport::HTTP::Daemon);

#Package Globals (PUBLIC)
our $DynamicModuleList   = ('SOAP::Transport::HTTP');
our $MaxChildren         = $config::aosi_max_children;
our $MaxRequestsPerChild = $config::aosi_max_requests;
our $DEBUG               = $config::aosi_debug_level;
our $ChildCountFile;

write_proc_log("====== Restarting =======");
write_proc_log("MaxChildren:    $MaxChildren");
write_proc_log("MaxReqPerChild: $MaxRequestsPerChild");

#Package Globals (PRIVATE)
our %children;
my $die;
my $exitsoon;
my $full_debug = 1;

sub write_proc_log {
    my($stg) = @_;
    my($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime(time);
    my(@mon_name, $ts);

    @mon_name = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
    $ts = sprintf("%s %02d %02d:%02d:%02d", $mon_name[$mon], $mday, $hour, $min, $sec);

    open(LOG, ">>$config::proc_log_file") or die "$0: can not open file $config::proc_log_file for appending ($!)\n";
    $stg =~ s/\n//gm;
    #$stg =~ s/\s\s+//g;
    print LOG $ts," [",$$,"]: ",AOSICFG::utfize($stg),"\n";
    close(LOG);
}


#Functions...
sub handle {
    my $self          = shift->new;
    my($var_ref, $timeout) = @_;      # DES - set timeout and get IP
    my $parentpid     = $$;
    my $oldchildcount = 0;

	write_proc_log("Initializing...") if $full_debug;
    $SIG{TERM} = sub {
        foreach (keys %children) { kill(15, $_) }
        $die++;
    };
    $SIG{HUP} = sub {
        foreach (keys %children) { kill(15, $_) }
        return 1;
    };
    $SIG{INT} = sub {
        foreach (keys %children) { kill(2, $_) }
        return 1;
    };
    $SIG{USR1} = sub { $MaxChildren++; };
    $SIG{USR2} = sub { $MaxChildren--; };
    $SIG{CHLD} = 'DEFAULT';
	write_proc_log("Done!") if $full_debug;

    while (!$die) {
		#write_proc_log("Main loop...") if $full_debug;
        if ($ChildCountFile && ($oldchildcount != $MaxChildren)) {
            #write_proc_log("Output new MaxChildren to $ChildCountFile") if $DEBUG;
            open CHILDCOUNT, "> $ChildCountFile" or next;
            print CHILDCOUNT $MaxChildren;
            close CHILDCOUNT;
            $oldchildcount = $MaxChildren;
        }

        if (scalar(keys %children) < $MaxChildren) {
			#write_proc_log("Creating child...") if $full_debug;
            my $childpid;
            if ($childpid = fork) {    # parent
                $children{$childpid} = 1;
                write_proc_log("  Child created: $childpid") if $DEBUG;
            }
            else {                     #child
				#write_proc_log("In child...") if $full_debug;
                $SIG{TERM} = sub { exit };
                $SIG{INT} = sub { $exitsoon++; return 1 };
                #$SIG{PIPE} = 'IGNORE';
                $SIG{PIPE} = sub { write_proc_log("Caught SIGPIPE, exiting..."); exit };
                foreach my $module (@DynamicModuleList) {
                    eval "require $module";
                    print $@ if $@;
                }
                my $counter = 1;
                while (my $c = $self->accept) {
					#write_proc_log("self: ".$self."\nc: ".$c."\n");
                    $$var_ref = $c->peerhost;       # DES - get IP
					write_proc_log("socket: ".$c->sockhost.":".$c->sockport) if $DEBUG;
					write_proc_log("peer:   ".$c->peerhost.":".$c->peerport) if $DEBUG;
                    $c->timeout($timeout);          # DES - set timeout
                    while (my $r = $c->get_request) {
                        if ($r->method eq 'GET' and $r->url->path eq "/wsdl") {
							#write_proc_log("Sending wsdl!") if $full_debug;
                           	$c->send_file_response($config::wsdl_file);
                        }
                        else {
							#write_proc_log("Sending SOAP!") if $full_debug;
                            $self->request($r);
                            $self->SOAP::Transport::HTTP::Server::handle;
                            $c->send_response($self->response);
                        }
                    }
					#write_proc_log("Before closing c: ".$c) if $full_debug;
                    $c->close;
                    undef $c;
                    write_proc_log("  Child ".$$." handled request # $counter") if $DEBUG;
                    exit                                        if $exitsoon;
                    exit                                        if ++$counter > $MaxRequestsPerChild;
                    exit if !kill(0, $parentpid);
                }
                exit;
            }
        }

        $kid = waitpid(-1, &WNOHANG);
        delete $children{$kid} if ($kid > 0);
        write_proc_log("  Reaped child: $kid") if (($kid > 0) && $DEBUG);
        sleep 1;
    }
}

1;

